/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2014 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::localTranslatingFixedValueFvPatchField

Group
    grpGenericBoundaryConditions

Description
    This boundary condition provides an localTranslating condition in terms of
    amplitude and frequency.

        \f[
            x_p = (1 + a sin(2 \pi f t))x_{ref} + x_o
        \f]

    where

    \vartable
        x_p     | patch values
        x_{ref} | patch reference values
        x_o     | patch offset values
        a       | amplitude
        f       | frequency [1/s]
        t       | time [s]
    \endvartable

    \heading Patch usage

    \table
        Property     | Description             | Required    | Default value
        refValue     | reference value         | yes         |
        offset       | offset value            | no          | 0.0
        amplitude    | oscillation amplitude   | yes         |
        frequency    | oscillation frequency   | yes         |
    \endtable

    Example of the boundary condition specification:
    \verbatim
    myPatch
    {
        type            localTranslatingFixedValue;
        refValue        uniform 5.0;
        offset          0.0;
        amplitude       constant 0.5;
        frequency       constant 10;
    }
    \endverbatim

Note
    The amplitude and frequency entries are DataEntry types, able to describe
    time varying functions.  The example above gives the usage for supplying
    constant values.

SeeAlso
    Foam::DataEntry

SourceFiles
    localTranslatingFixedValueFvPatchField.C

\*---------------------------------------------------------------------------*/

#ifndef localTranslatingFixedValueFvPatchField_H
#define localTranslatingFixedValueFvPatchField_H

#include "Random.H"
#include "fixedValueFvPatchFields.H"
#include "DataEntry.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
              Class localTranslatingFixedValueFvPatchField Declaration
\*---------------------------------------------------------------------------*/

template<class Type>
class localTranslatingFixedValueFvPatchField
:
    public fixedValueFvPatchField<Type>
{
    // Private data

        //- Value of the field at the dropInlet
        Field<Type> dropInletValue_;

        //- Value of the field outside the dropInlet
        Field<Type> dropOutsideValue_;

	//- Initial Centres of the dropInlet
	 List<vector> dropCentre0_;			//Centre of droplet at t = 0;

	//- Centres of the dropInlet at time t	
	  List<vector> dropCentre_;			//Centre of droplet at time t;


	//- Radii of the dropInlet		//If more than 1 dropInlet, List all radius
	  List<scalar> dropRadius_;


	//Face centers
	Field<vector> faceCentres_;

        //- Current time index
        label curTimeIndex_;


	//- Translating velocity of the droplet	
	List<vector> dropTranslatingVelocity_;

	//- Frequency of the droplet 
	scalar dropFrequency_;


//in order to update dropCentre. May be this is the key

    // Private Member Functions

        //- Return current scale
        //scalar currentScale() const;
        vector currentScale() const;


public:

    //- Runtime type information
    TypeName("localTranslatingFixedValue");


    // Constructors

        //- Construct from patch and internal field
        localTranslatingFixedValueFvPatchField
        (
            const fvPatch&,
            const DimensionedField<Type, volMesh>&
        );

        //- Construct from patch, internal field and dictionary
        localTranslatingFixedValueFvPatchField
        (
            const fvPatch&,
            const DimensionedField<Type, volMesh>&,
            const dictionary&
        );

        //- Construct by mapping given localTranslatingFixedValueFvPatchField
        //  onto a new patch
        localTranslatingFixedValueFvPatchField
        (
            const localTranslatingFixedValueFvPatchField<Type>&,
            const fvPatch&,
            const DimensionedField<Type, volMesh>&,
            const fvPatchFieldMapper&
        );

        //- Construct as copy
        localTranslatingFixedValueFvPatchField
        (
            const localTranslatingFixedValueFvPatchField<Type>&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchField<Type> > clone() const
        {
            return tmp<fvPatchField<Type> >
            (
                new localTranslatingFixedValueFvPatchField<Type>(*this)
            );
        }

        //- Construct as copy setting internal field reference
        localTranslatingFixedValueFvPatchField
        (
            const localTranslatingFixedValueFvPatchField<Type>&,
            const DimensionedField<Type, volMesh>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchField<Type> > clone
        (
            const DimensionedField<Type, volMesh>& iF
        ) const
        {
            return tmp<fvPatchField<Type> >
            (
                new localTranslatingFixedValueFvPatchField<Type>(*this, iF)
            );
        }


    // Member functions

        // Access

            //- Return the dropInlet value
            const Field<Type>& dropInletValue() const
            {
                return dropInletValue_;
            }

            //- Return reference to the dropInlet value to allow adjustment
            Field<Type>& dropInletValue()
            {
                return dropInletValue_;
            }



            //- Return the dropOutside value
            const Field<Type>& dropOutsideValue() const
            {
                return dropOutsideValue_;
            }

            //- Return reference to the dropOutside value to allow adjustment
            Field<Type>& dropOutsideValue()
            {
                return dropOutsideValue_;
            }

	//- initial dropcentre
	List<vector> dropCentre0() const
	{
	    return dropCentre0_;
	}
	List<vector>& dropCentre0()
	{
	    return dropCentre0_;
	}


	//-
	List<vector> dropCentre() const
	{
	    return dropCentre_;
	}
	List<vector>& dropCentre()
	{
	    return dropCentre_;
	}


	//-
	List<scalar> dropRadius() const
	{
	    return dropRadius_;
	}
	List<scalar>& dropRadius()
	{
	    return dropRadius_;
	}


	//-
	Field<vector> faceCentres() const
	{
	    return faceCentres_;
	}
	Field<vector>& faceCentres()
	{
	    return faceCentres_;
	}



	//- Translating velocity of the droplet		//To use later
	Field<vector> dropTranslatingVelocity() const
	{
	    return dropTranslatingVelocity_;
	}
	Field<vector>& dropTranslatingVelocity()
	{
	    return dropTranslatingVelocity_;
	}



    scalar& dropFrequency()
    {
          return dropFrequency_;
    }




        // Mapping functions- //Remove the mapping function for now.


            //- Map (and resize as needed) from self given a mapping object
            virtual void autoMap
            (
                const fvPatchFieldMapper&
            );

            //- Reverse map the given fvPatchField onto this fvPatchField
            virtual void rmap
            (
                const fvPatchField<Type>&,
                const labelList&
            );


        // Evaluation functions

            //- Update the coefficients associated with the patch field
            virtual void updateCoeffs();


        //- Write
        virtual void write(Ostream&) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#ifdef NoRepository
#   include "localTranslatingFixedValueFvPatchField.C"
#endif

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
